/**
 * IMPORTANT: Your life will be much easier if you open this stylesheet's
 * containing .biketheme folder in a code editor (such as VSCode). Then you will
 * have type checking and autocomplete as you edit this file.
 *
 * Also Bike will live-repload your outline when you save edits to this file. If
 * you want to play with themes first make sure you have autocomplete working,
 * second make sure you have live reloading working. Life will be easier!
 */

/**
 * Bike Stylesheets README
 *
 * Bike stylesheets are similar in _concept_ to CSS stylesheets, but the
 * implementation is completely different. Some big differences between Bike
 * stylesheets and CSS stylesheets are:
 *
 * 1. Bike's are written in JavaScript, not CSS
 * 2. Bike's use Bike path syntax to match, not CSS selectors
 * 3. Bike has three different style targets: branch, row, and run.
 * 4. Bike styling uses some similar terms to CSS, but generally Bike is it's
 *    own more focused system designed directly to Bike's needs.
 *
 * RELATIVE MATCH PATHS: The match paths should all be relative (starting with a
 * .). When Bike is styling a branch, row, or run, it matches if the path
 * matches relative to that element. More on outline path syntax:
 * https://bikeguide.hogbaysoftware.com/using-bike/using-outline-paths
 */

//@ts-check

var Types = require("./types"); // Hardcoded, require() calls don't generally work!
var defineBranchRule = Types.defineBranchRule;
var defineRowRule = Types.defineRowRule;
var defineRunRule = Types.defineRunRule;
var Insets = Types.Insets;
var Color = Types.Color;
var Font = Types.Font;

/**
 * Branch Rules
 */

defineBranchRule(".*", (env, branch) => {
    branch.background.padding = new Insets(0, 0, 0, 28)
})
defineBranchRule(".isRootRow(.) = true", (env, branch) => {
    branch.rows.font = Font.systemBody()
    branch.rows.lineHeightMultiple = 1.3
    branch.rows.color = env.foregroundColor
    branch.rows.contentBackground.color = env.backgroundColor
    branch.rows.contentBackground.padding = new Insets(0, 0, 0, 0)
})
defineBranchRule(".isFocusedRow(.) = false", (env, branch) => {
    branch.background.opacity = 0
})

/**
 * Row Styles
 */

defineRowRule(".isFocusedRow(.) = false", (env, row) => {
    row.opacity = 0.1
    row.contentBackground.opacity = 0
})
defineRowRule(".isBlockSelectionRow(.) = true", (env, row) => {
    row.color = Color.white()
    row.contentBackground.color = env.accentColor
})
defineRowRule(".heading", (env, row) => {
    row.font = row.font.withBold()
}),
defineRowRule(".codeblock", (env, row) => {
    row.font = row.font.withMonospace()
}),
defineRowRule(".blockquote", (env, row) => {
    row.font = row.font.withItalics()
}),
defineRowRule(".note", (env, row) => {
    row.font = row.font.withItalics()
    row.contentBackground.borderWidth = 1
    row.contentBackground.borderColor = Color.systemGray()
})

/**
 * Model Run Styles
 */

defineRunRule(".@emphasized", (env, run) => {
    run.font = run.font.withItalics()
}),
defineRunRule(".@strong", (env, run) => {
    run.font = run.font.withBold()
}),
defineRunRule(".@code", (env, run) => {
    run.font = run.font.withMonospace()
}),
defineRunRule(".@highlight", (env, run) => {
    run.backgroundColor = Color.systemYellow()
}),
defineRunRule(".@strikethrough", (env, run) => {
    run.textDecorations.strikethrough.thick = true
}),
defineRunRule(".@link", (env, run) => {
    run.color = Color.systemBlue()
    run.textDecorations.underline.thick = true
}),
defineRunRule(".@baseline = subscript", (env, run) => {
    run.baselineOffset = run.font.size * -0.25
    run.font.size *= 0.75
}),
defineRunRule(".@baseline = superscript", (env, run) => {
    run.baselineOffset = run.font.size * 0.25
    run.font.size *= 0.75
})

/**
 * View Run Styles
 */

defineRunRule(".@view-selected-range", (env, run) => {
    run.backgroundColor = env.accentColor.withFraction(0.77, env.backgroundColor)
})
